@@ -34,115 +34,15 @@
 #include <stdlib.h>
 #include <limits.h>
 #include <string.h>
+#include <syslog.h>
 #include <assert.h>
 #include <X11/Xatom.h>
 #include <X11/Xlib.h>
-#include <X11/extensions/Xrandr.h>
-#include <X11/extensions/Xinerama.h>
 #include <X11/extensions/Xfixes.h>
 #include "vdagentd-proto.h"
 #include "vdagent-x11.h"
+#include "vdagent-x11-priv.h"
 
-/* Macros to print a message to the logfile prefixed by the selection */
-#define SELPRINTF(format, ...) \
-    fprintf(x11->errfile, "%s: " format, \
-            vdagent_x11_sel_to_str(selection), ##__VA_ARGS__)
-
-#define VSELPRINTF(format, ...) \
-    do { \
-        if (x11->verbose) { \
-            fprintf(x11->errfile, "%s: " format, \
-                    vdagent_x11_sel_to_str(selection), ##__VA_ARGS__); \
-        } \
-    } while (0)
-
-enum { owner_none, owner_guest, owner_client };
-
-/* X11 terminology is confusing a selection request is a request from an
-   app to get clipboard data from us, so iow from the spice client through
-   the vdagent channel. We handle these one at a time and queue any which
-   come in while we are still handling the current one. */
-struct vdagent_x11_selection_request {
-    XEvent event;
-    uint8_t selection;
-    struct vdagent_x11_selection_request *next;
-};
-
-/* A conversion request is X11 speak for asking an other app to give its
-   clipboard data to us, we do these on behalf of the spice client to copy
-   data from the guest to the client. Like selection requests we process
-   these one at a time. */
-struct vdagent_x11_conversion_request {
-    Atom target;
-    uint8_t selection;
-    struct vdagent_x11_conversion_request *next;
-};
-
-struct clipboard_format_tmpl {
-    uint32_t type;
-    const char *atom_names[16];
-};
-
-struct clipboard_format_info {
-    uint32_t type;
-    Atom atoms[16];
-    int atom_count;
-};
-
-static const struct clipboard_format_tmpl clipboard_format_templates[] = {
-    { VD_AGENT_CLIPBOARD_UTF8_TEXT, { "UTF8_STRING",
-      "text/plain;charset=UTF-8", "text/plain;charset=utf-8", NULL }, },
-    { VD_AGENT_CLIPBOARD_IMAGE_PNG, { "image/png", NULL }, },
-    { VD_AGENT_CLIPBOARD_IMAGE_BMP, { "image/bmp", "image/x-bmp",
-      "image/x-MS-bmp", "image/x-win-bitmap", NULL }, },
-    { VD_AGENT_CLIPBOARD_IMAGE_TIFF, { "image/tiff", NULL }, },
-    { VD_AGENT_CLIPBOARD_IMAGE_JPG, { "image/jpeg", NULL }, },
-};
-
-#define clipboard_format_count (sizeof(clipboard_format_templates)/sizeof(clipboard_format_templates[0]))
-
-struct vdagent_x11 {
-    struct clipboard_format_info clipboard_formats[clipboard_format_count];
-    Display *display;
-    Atom clipboard_atom;
-    Atom clipboard_primary_atom;
-    Atom targets_atom;
-    Atom incr_atom;
-    Atom multiple_atom;
-    Window root_window;
-    Window selection_window;
-    struct udscs_connection *vdagentd;
-    FILE *errfile;
-    int verbose;
-    int fd;
-    int screen;
-    int width;
-    int height;
-    int has_xrandr;
-    int has_xinerama;
-    int has_xfixes;
-    int xfixes_event_base;
-    int max_prop_size;
-    int expected_targets_notifies[256];
-    int clipboard_owner[256];
-    int clipboard_type_count[256];
-    uint32_t clipboard_agent_types[256][256];
-    Atom clipboard_x11_targets[256][256];
-    /* Data for conversion_req which is currently being processed */
-    struct vdagent_x11_conversion_request *conversion_req;
-    int expect_property_notify;
-    uint8_t *clipboard_data;
-    uint32_t clipboard_data_size;
-    uint32_t clipboard_data_space;
-    /* Data for selection_req which is currently being processed */
-    struct vdagent_x11_selection_request *selection_req;
-    uint8_t *selection_req_data;
-    uint32_t selection_req_data_pos;
-    uint32_t selection_req_data_size;
-    Atom selection_req_atom;
-};
-
-static void vdagent_x11_send_daemon_guest_xorg_res(struct vdagent_x11 *x11);
 static void vdagent_x11_handle_selection_notify(struct vdagent_x11 *x11,
                                                 XEvent *event, int incr);
 static void vdagent_x11_handle_selection_request(struct vdagent_x11 *x11);
@@ -168,8 +68,13 @@ static const char *vdagent_x11_sel_to_str(uint8_t selection) {
     }
 }
 
+static int debug_error_handler(Display *display, XErrorEvent *error)
+{
+    abort();
+}
+
 struct vdagent_x11 *vdagent_x11_create(struct udscs_connection *vdagentd,
-    FILE *errfile, int verbose)
+    int debug, int sync)
 {
     struct vdagent_x11 *x11;
     XWindowAttributes attrib;
@@ -177,21 +82,25 @@ struct vdagent_x11 *vdagent_x11_create(struct udscs_connection *vdagentd,
 
     x11 = calloc(1, sizeof(*x11));
     if (!x11) {
-        fprintf(errfile, "out of memory allocating vdagent_x11 struct\n");
+        syslog(LOG_ERR, "out of memory allocating vdagent_x11 struct");
         return NULL;
     }
 
     x11->vdagentd = vdagentd;
-    x11->errfile = errfile;
-    x11->verbose = verbose;
+    x11->debug = debug;
 
     x11->display = XOpenDisplay(NULL);
     if (!x11->display) {
-        fprintf(x11->errfile, "could not connect to X-server\n");
+        syslog(LOG_ERR, "could not connect to X-server");
         free(x11);
         return NULL;
     }
 
+    if (sync) {
+        XSetErrorHandler(debug_error_handler);
+        XSynchronize(x11->display, True);
+    }
+
     x11->screen = DefaultScreen(x11->display);
     x11->root_window = RootWindow(x11->display, x11->screen);
     x11->fd = ConnectionNumber(x11->display);
@@ -214,31 +123,10 @@ struct vdagent_x11 *vdagent_x11_create(struct udscs_connection *vdagentd,
     /* We should not store properties (for selections) on the root window */
     x11->selection_window = XCreateSimpleWindow(x11->display, x11->root_window,
                                                 0, 0, 1, 1, 0, 0, 0);
-    if (x11->verbose)
-        fprintf(x11->errfile, "Selection window: %u\n",
-                (unsigned int)x11->selection_window);
+    if (x11->debug)
+        syslog(LOG_DEBUG, "Selection window: %u", (int)x11->selection_window);
 
-    if (XRRQueryExtension(x11->display, &i, &i))
-        x11->has_xrandr = 1;
-
-    if (XineramaQueryExtension(x11->display, &i, &i))
-        x11->has_xinerama = 1;
-
-    switch (x11->has_xrandr << 4 | x11->has_xinerama) {
-    case 0x00:
-        fprintf(x11->errfile, "Neither Xrandr nor Xinerama found, assuming single monitor setup\n");
-        break;
-    case 0x01:
-        if (x11->verbose)
-            fprintf(x11->errfile, "Found Xinerama extension without Xrandr, assuming a multi monitor setup\n");
-        break;
-    case 0x10:
-        fprintf(x11->errfile, "Found Xrandr but no Xinerama, weird! Assuming a single monitor setup\n");
-        break;
-    case 0x11:
-        /* Standard single monitor setup, nothing to see here */
-        break;
-    }
+    vdagent_x11_randr_init(x11);
 
     if (XFixesQueryExtension(x11->display, &x11->xfixes_event_base, &i) &&
         XFixesQueryVersion(x11->display, &major, &minor) && major >= 1) {
@@ -254,8 +142,7 @@ struct vdagent_x11 *vdagent_x11_create(struct udscs_connection *vdagentd,
                                    XFixesSelectionWindowDestroyNotifyMask|
                                    XFixesSelectionClientCloseNotifyMask);
     } else
-        fprintf(x11->errfile,
-                "no xfixes, no guest -> client copy paste support\n");
+        syslog(LOG_ERR, "no xfixes, no guest -> client copy paste support");
 
     x11->max_prop_size = XExtendedMaxRequestSize(x11->display);
     if (x11->max_prop_size) {
@@ -335,7 +222,7 @@ static void vdagent_x11_set_clipboard_owner(struct vdagent_x11 *x11,
         if (curr_sel->selection == selection) {
             if (once) {
                 SELPRINTF("selection requests pending on clipboard ownership "
-                          "change, clearing\n");
+                          "change, clearing");
                 once = 0;
             }
             vdagent_x11_send_selection_notify(x11, None, curr_sel);
@@ -364,7 +251,7 @@ static void vdagent_x11_set_clipboard_owner(struct vdagent_x11 *x11,
         if (curr_conv->selection == selection) {
             if (once) {
                 SELPRINTF("client clipboard request pending on clipboard "
-                          "ownership change, clearing\n");
+                          "ownership change, clearing");
                 once = 0;
             }
             udscs_write(x11->vdagentd, VDAGENTD_CLIPBOARD_DATA, selection,
@@ -401,7 +288,7 @@ static int vdagent_x11_get_clipboard_atom(struct vdagent_x11 *x11, uint8_t selec
     } else if (selection == VD_AGENT_CLIPBOARD_SELECTION_PRIMARY) {
         *clipboard = x11->clipboard_primary_atom;
     } else {
-        fprintf(x11->errfile, "get_clipboard_atom: unknown selection\n");
+        syslog(LOG_ERR, "get_clipboard_atom: unknown selection");
         return -1;
     }
 
@@ -421,7 +308,7 @@ static int vdagent_x11_get_clipboard_selection(struct vdagent_x11 *x11,
     } else if (event->type == SelectionRequest) {
         atom = event->xselectionrequest.selection;
     } else {
-        fprintf(x11->errfile, "get_clipboard_selection: unknown event type\n");
+        syslog(LOG_ERR, "get_clipboard_selection: unknown event type");
         return -1;
     }
 
@@ -430,7 +317,7 @@ static int vdagent_x11_get_clipboard_selection(struct vdagent_x11 *x11,
     } else if (atom == x11->clipboard_primary_atom) {
         *selection = VD_AGENT_CLIPBOARD_SELECTION_PRIMARY;
     } else {
-        fprintf(x11->errfile, "get_clipboard_selection: unknown selection\n");
+        syslog(LOG_ERR, "get_clipboard_selection: unknown selection");
         return -1;
     }
 
@@ -462,11 +349,11 @@ static void vdagent_x11_handle_event(struct vdagent_x11 *x11, XEvent event)
             ev.xfev.owner = None;
             break;
         default:
-            VSELPRINTF("unexpected xfix event subtype %d window %d\n",
+            VSELPRINTF("unexpected xfix event subtype %d window %d",
                        (int)ev.xfev.subtype, (int)event.xany.window);
             return;
         }
-        VSELPRINTF("New selection owner: %u\n", (unsigned int)ev.xfev.owner);
+        VSELPRINTF("New selection owner: %u", (unsigned int)ev.xfev.owner);
 
         /* Ignore becoming the owner ourselves */
         if (ev.xfev.owner == x11->selection_window)
@@ -488,19 +375,13 @@ static void vdagent_x11_handle_event(struct vdagent_x11 *x11, XEvent event)
 
     switch (event.type) {
     case ConfigureNotify:
+        // TODO: handle CrtcConfigureNotify, OutputConfigureNotify can be ignored.
         if (event.xconfigure.window != x11->root_window)
             break;
 
         handled = 1;
-
-        if (event.xconfigure.width  == x11->width &&
-            event.xconfigure.height == x11->height)
-            break;
-
-        x11->width  = event.xconfigure.width;
-        x11->height = event.xconfigure.height;
-
-        vdagent_x11_send_daemon_guest_xorg_res(x11);
+        vdagent_x11_randr_handle_root_size_change(x11,
+                event.xconfigure.width, event.xconfigure.height);
         break;
     case MappingNotify:
         /* These are uninteresting */
@@ -541,7 +422,7 @@ static void vdagent_x11_handle_event(struct vdagent_x11 *x11, XEvent event)
 
         new_req = malloc(sizeof(*new_req));
         if (!new_req) {
-            SELPRINTF("out of memory on SelectionRequest, ignoring.\n");
+            SELPRINTF("out of memory on SelectionRequest, ignoring.");
             break;
         }
 
@@ -566,9 +447,9 @@ static void vdagent_x11_handle_event(struct vdagent_x11 *x11, XEvent event)
         break;
     }
     }
-    if (!handled && x11->verbose)
-        fprintf(x11->errfile, "unhandled x11 event, type %d, window %d\n",
-                (int)event.type, (int)event.xany.window);
+    if (!handled && x11->debug)
+        syslog(LOG_DEBUG, "unhandled x11 event, type %d, window %d",
+               (int)event.type, (int)event.xany.window);
 }
 
 void vdagent_x11_do_read(struct vdagent_x11 *x11)
@@ -581,53 +462,6 @@ void vdagent_x11_do_read(struct vdagent_x11 *x11)
     }
 }
 
-static void vdagent_x11_send_daemon_guest_xorg_res(struct vdagent_x11 *x11)
-{
-    struct vdagentd_guest_xorg_resolution *res = NULL;
-    XineramaScreenInfo *screen_info = NULL;
-    int i, screen_count = 0;
-
-    if (x11->has_xinerama)
-        screen_info = XineramaQueryScreens(x11->display, &screen_count);
-
-    if (screen_count == 0)
-        screen_count = 1;
-
-    res = malloc(screen_count * sizeof(*res));
-    if (!res) {
-        fprintf(x11->errfile, "out of memory while trying to send resolutions, not sending resolutions.\n");
-        if (screen_info)
-            XFree(screen_info);
-        return;
-    }
-
-    if (screen_info) {
-        for (i = 0; i < screen_count; i++) {
-            if (screen_info[i].screen_number >= screen_count) {
-                fprintf(x11->errfile, "Invalid screen number in xinerama screen info (%d >= %d)\n",
-                        screen_info[i].screen_number, screen_count);
-                XFree(screen_info);
-                free(res);
-                return;
-            }
-            res[screen_info[i].screen_number].width = screen_info[i].width;
-            res[screen_info[i].screen_number].height = screen_info[i].height;
-            res[screen_info[i].screen_number].x = screen_info[i].x_org;
-            res[screen_info[i].screen_number].y = screen_info[i].y_org;
-        }
-        XFree(screen_info);
-    } else {
-        res[0].width  = x11->width;
-        res[0].height = x11->height;
-        res[0].x = 0;
-        res[0].y = 0;
-    }
-
-    udscs_write(x11->vdagentd, VDAGENTD_GUEST_XORG_RESOLUTION, x11->width,
-                x11->height, (uint8_t *)res, screen_count * sizeof(*res));
-    free(res);
-}
-
 static const char *vdagent_x11_get_atom_name(struct vdagent_x11 *x11, Atom a)
 {
     if (a == None)
@@ -650,13 +484,13 @@ static int vdagent_x11_get_selection(struct vdagent_x11 *x11, XEvent *event,
 
     if (!incr) {
         if (event->xselection.property == None) {
-            VSELPRINTF("XConvertSelection refused by clipboard owner\n");
+            VSELPRINTF("XConvertSelection refused by clipboard owner");
             goto exit;
         }
 
         if (event->xselection.requestor != x11->selection_window ||
             event->xselection.property != prop) {
-            SELPRINTF("SelectionNotify parameters mismatch\n");
+            SELPRINTF("SelectionNotify parameters mismatch");
             goto exit;
         }
     }
@@ -664,7 +498,7 @@ static int vdagent_x11_get_selection(struct vdagent_x11 *x11, XEvent *event,
     if (XGetWindowProperty(x11->display, x11->selection_window, prop, 0,
                            LONG_MAX, del, type, &type_ret, &format_ret, &len,
                            &remain, &data) != Success) {
-        SELPRINTF("XGetWindowProperty failed\n");
+        SELPRINTF("XGetWindowProperty failed");
         goto exit;
     }
 
@@ -674,7 +508,7 @@ static int vdagent_x11_get_selection(struct vdagent_x11 *x11, XEvent *event,
 
             if (x11->expect_property_notify) {
                 SELPRINTF("received an incr SelectionNotify while "
-                          "still reading another incr property\n");
+                          "still reading another incr property");
                 goto exit;
             }
 
@@ -682,7 +516,7 @@ static int vdagent_x11_get_selection(struct vdagent_x11 *x11, XEvent *event,
                 free(x11->clipboard_data);
                 x11->clipboard_data = malloc(prop_min_size);
                 if (!x11->clipboard_data) {
-                    SELPRINTF("out of memory allocating clipboard buffer\n");
+                    SELPRINTF("out of memory allocating clipboard buffer");
                     x11->clipboard_data_space = 0;
                     goto exit;
                 }
@@ -699,14 +533,14 @@ static int vdagent_x11_get_selection(struct vdagent_x11 *x11, XEvent *event,
     }
 
     if (type_ret != type) {
-        SELPRINTF("expected property type: %s, got: %s\n",
+        SELPRINTF("expected property type: %s, got: %s",
                   vdagent_x11_get_atom_name(x11, type),
                   vdagent_x11_get_atom_name(x11, type_ret));
         goto exit;
     }
 
     if (format_ret != format) {
-        SELPRINTF("expected %d bit format, got %d bits\n", format, format_ret);
+        SELPRINTF("expected %d bit format, got %d bits", format, format_ret);
         goto exit;
     }
 
@@ -731,7 +565,7 @@ static int vdagent_x11_get_selection(struct vdagent_x11 *x11, XEvent *event,
                 x11->clipboard_data = realloc(x11->clipboard_data,
                                               x11->clipboard_data_space);
                 if (!x11->clipboard_data) {
-                    SELPRINTF("out of memory allocating clipboard buffer\n");
+                    SELPRINTF("out of memory allocating clipboard buffer");
                     x11->clipboard_data_space = 0;
                     free(old_clipboard_data);
                     goto exit;
@@ -739,7 +573,7 @@ static int vdagent_x11_get_selection(struct vdagent_x11 *x11, XEvent *event,
             }
             memcpy(x11->clipboard_data + x11->clipboard_data_size, data, len);
             x11->clipboard_data_size += len;
-            VSELPRINTF("Appended %ld bytes to buffer\n", len);
+            VSELPRINTF("Appended %ld bytes to buffer", len);
             XFree(data);
             return 0; /* Wait for more data */
         }
@@ -751,7 +585,7 @@ static int vdagent_x11_get_selection(struct vdagent_x11 *x11, XEvent *event,
     if (len > 0) {
         ret_val = len;
     } else {
-        SELPRINTF("property contains no data (zero length)\n");
+        SELPRINTF("property contains no data (zero length)");
         *data_ret = NULL;
     }
 
@@ -794,7 +628,7 @@ static uint32_t vdagent_x11_target_to_type(struct vdagent_x11 *x11,
         }
     }
 
-    SELPRINTF("unexpected selection type %s\n",
+    SELPRINTF("unexpected selection type %s",
               vdagent_x11_get_atom_name(x11, target));
     return VD_AGENT_CLIPBOARD_NONE;
 }
@@ -809,13 +643,13 @@ static Atom vdagent_x11_type_to_target(struct vdagent_x11 *x11,
             return x11->clipboard_x11_targets[selection][i];
         }
     }
-    SELPRINTF("client requested unavailable type %u\n", type);
+    SELPRINTF("client requested unavailable type %u", type);
     return None;
 }
 
 static void vdagent_x11_handle_conversion_request(struct vdagent_x11 *x11)
 {
-    Atom clip;
+    Atom clip = None;
 
     if (!x11->conversion_req) {
         return;
@@ -833,10 +667,10 @@ static void vdagent_x11_handle_selection_notify(struct vdagent_x11 *x11,
     unsigned char *data = NULL;
     uint32_t type;
     uint8_t selection = -1;
-    Atom clip;
+    Atom clip = None;
 
     if (!x11->conversion_req) {
-        fprintf(x11->errfile, "SelectionNotify received without a target\n");
+        syslog(LOG_ERR, "SelectionNotify received without a target");
         return;
     }
     vdagent_x11_get_clipboard_atom(x11, x11->conversion_req->selection, &clip);
@@ -850,13 +684,13 @@ static void vdagent_x11_handle_selection_notify(struct vdagent_x11 *x11,
         if (vdagent_x11_get_clipboard_selection(x11, event, &selection)) {
             len = -1;
         } else if (selection != x11->conversion_req->selection) {
-            SELPRINTF("Requested data for selection %d got %d\n",
+            SELPRINTF("Requested data for selection %d got %d",
                       (int)x11->conversion_req->selection, (int)selection);
             len = -1;
         }
         if (event->xselection.target != x11->conversion_req->target &&
                 event->xselection.target != x11->incr_atom) {
-            SELPRINTF("Requested %s target got %s\n",
+            SELPRINTF("Requested %s target got %s",
                 vdagent_x11_get_atom_name(x11, x11->conversion_req->target),
                 vdagent_x11_get_atom_name(x11, event->xselection.target));
             len = -1;
@@ -903,9 +737,9 @@ static void vdagent_x11_print_targets(struct vdagent_x11 *x11,
     uint8_t selection, const char *action, Atom *atoms, int c)
 {
     int i;
-    VSELPRINTF("%s %d targets:\n", action, c);
+    VSELPRINTF("%s %d targets:", action, c);
     for (i = 0; i < c; i++)
-        VSELPRINTF("%s\n", vdagent_x11_get_atom_name(x11, atoms[i]));
+        VSELPRINTF("%s", vdagent_x11_get_atom_name(x11, atoms[i]));
 }
 
 static void vdagent_x11_handle_targets_notify(struct vdagent_x11 *x11,
@@ -921,7 +755,7 @@ static void vdagent_x11_handle_targets_notify(struct vdagent_x11 *x11,
     }
 
     if (!x11->expected_targets_notifies[selection]) {
-        SELPRINTF("unexpected selection notify TARGETS\n");
+        SELPRINTF("unexpected selection notify TARGETS");
         return;
     }
 
@@ -956,7 +790,7 @@ static void vdagent_x11_handle_targets_notify(struct vdagent_x11 *x11,
             (*type_count)++;
             if (*type_count ==
                     sizeof(x11->clipboard_agent_types[0])/sizeof(uint32_t)) {
-                SELPRINTF("handle_targets_notify: too many types\n");
+                SELPRINTF("handle_targets_notify: too many types");
                 break;
             }
         }
@@ -1014,7 +848,7 @@ static void vdagent_x11_send_targets(struct vdagent_x11 *x11,
                 targets[target_count] = x11->clipboard_formats[j].atoms[k];
                 target_count++;
                 if (target_count == sizeof(targets)/sizeof(Atom)) {
-                    SELPRINTF("send_targets: too many targets\n");
+                    SELPRINTF("send_targets: too many targets");
                     goto exit_loop;
                 }
             }
@@ -1047,14 +881,14 @@ static void vdagent_x11_handle_selection_request(struct vdagent_x11 *x11)
 
     if (x11->clipboard_owner[selection] != owner_client) {
         SELPRINTF("received selection request event for target %s, "
-                  "while not owning client clipboard\n",
+                  "while not owning client clipboard",
             vdagent_x11_get_atom_name(x11, event->xselectionrequest.target));
         vdagent_x11_send_selection_notify(x11, None, NULL);
         return;
     }
 
     if (event->xselectionrequest.target == x11->multiple_atom) {
-        SELPRINTF("multiple target not supported\n");
+        SELPRINTF("multiple target not supported");
         vdagent_x11_send_selection_notify(x11, None, NULL);
         return;
     }
@@ -1096,12 +930,12 @@ static void vdagent_x11_handle_property_delete_notify(struct vdagent_x11 *x11,
     }
 
     if (len) {
-        VSELPRINTF("Sending %d-%d/%d bytes of clipboard data\n",
+        VSELPRINTF("Sending %d-%d/%d bytes of clipboard data",
                 x11->selection_req_data_pos,
                 x11->selection_req_data_pos + len - 1,
                 x11->selection_req_data_size);
     } else {
-        VSELPRINTF("Ending incr send of clipboard data\n");
+        VSELPRINTF("Ending incr send of clipboard data");
     }
     XChangeProperty(x11->display, sel_event->xselectionrequest.requestor,
                     x11->selection_req_atom,
@@ -1124,67 +958,6 @@ static void vdagent_x11_handle_property_delete_notify(struct vdagent_x11 *x11,
     }
 }
 
-void vdagent_x11_set_monitor_config(struct vdagent_x11 *x11,
-                                    VDAgentMonitorsConfig *mon_config)
-{
-    int i, num_sizes = 0;
-    int best = -1;
-    unsigned int closest_diff = -1;
-    XRRScreenSize* sizes;
-    XRRScreenConfiguration* config;
-    Rotation rotation;
-
-    if (!x11->has_xrandr)
-        return;
-
-    if (mon_config->num_of_monitors != 1) {
-        fprintf(x11->errfile,
-                "Only 1 monitor supported, ignoring additional monitors\n");
-    }
-
-    sizes = XRRSizes(x11->display, x11->screen, &num_sizes);
-    if (!sizes || !num_sizes) {
-        fprintf(x11->errfile, "XRRSizes failed\n");
-        return;
-    }
-
-    /* Find the closest size which will fit within the monitor */
-    for (i = 0; i < num_sizes; i++) {
-        if (sizes[i].width  > mon_config->monitors[0].width ||
-            sizes[i].height > mon_config->monitors[0].height)
-            continue; /* Too large for the monitor */
-
-        unsigned int wdiff = mon_config->monitors[0].width  - sizes[i].width;
-        unsigned int hdiff = mon_config->monitors[0].height - sizes[i].height;
-        unsigned int diff = wdiff * wdiff + hdiff * hdiff;
-        if (diff < closest_diff) {
-            closest_diff = diff;
-            best = i;
-        }
-    }
-
-    if (best == -1) {
-        fprintf(x11->errfile, "no suitable resolution found for monitor\n");
-        return;
-    }
-
-    config = XRRGetScreenInfo(x11->display, x11->root_window);
-    if(!config) {
-        fprintf(x11->errfile, "get screen info failed\n");
-        return;
-    }
-    XRRConfigCurrentConfiguration(config, &rotation);
-    XRRSetScreenConfig(x11->display, config, x11->root_window, best,
-                       rotation, CurrentTime);
-    XRRFreeScreenConfigInfo(config);
-    x11->width = sizes[best].width;
-    x11->height = sizes[best].height;
-    vdagent_x11_send_daemon_guest_xorg_res(x11);
-
-    /* Flush output buffers and consume any pending events */
-    vdagent_x11_do_read(x11);
-}
-
 void vdagent_x11_clipboard_request(struct vdagent_x11 *x11,
         uint8_t selection, uint32_t type)
 {
@@ -1198,7 +971,7 @@ void vdagent_x11_clipboard_request(struct vdagent_x11 *x11,
     }
 
     if (x11->clipboard_owner[selection] != owner_guest) {
-        SELPRINTF("received clipboard req while not owning guest clipboard\n");
+        SELPRINTF("received clipboard req while not owning guest clipboard");
         goto none;
     }
 
@@ -1209,7 +982,7 @@ void vdagent_x11_clipboard_request(struct vdagent_x11 *x11,
 
     new_req = malloc(sizeof(*new_req));
     if (!new_req) {
-        SELPRINTF("out of memory on client clipboard request, ignoring.\n");
+        SELPRINTF("out of memory on client clipboard request, ignoring.");
         return;
     }
 
@@ -1241,14 +1014,14 @@ none:
 void vdagent_x11_clipboard_grab(struct vdagent_x11 *x11, uint8_t selection,
     uint32_t *types, uint32_t type_count)
 {
-    Atom clip;
+    Atom clip = None;
 
     if (vdagent_x11_get_clipboard_atom(x11, selection, &clip)) {
         return;
     }
 
     if (type_count > sizeof(x11->clipboard_agent_types[0])/sizeof(uint32_t)) {
-        SELPRINTF("x11_clipboard_grab: too many types\n");
+        SELPRINTF("x11_clipboard_grab: too many types");
         type_count = sizeof(x11->clipboard_agent_types[0])/sizeof(uint32_t);
     }
 
@@ -1274,7 +1047,7 @@ void vdagent_x11_clipboard_data(struct vdagent_x11 *x11, uint8_t selection,
     if (x11->selection_req_data) {
         if (type || size) {
             SELPRINTF("received clipboard data while still sending"
-                      " data from previous request, ignoring\n");
+                      " data from previous request, ignoring");
         }
         free(data);
         return;
@@ -1283,7 +1056,7 @@ void vdagent_x11_clipboard_data(struct vdagent_x11 *x11, uint8_t selection,
     if (!x11->selection_req) {
         if (type || size) {
             SELPRINTF("received clipboard data without an "
-                      "outstanding selection request, ignoring\n");
+                      "outstanding selection request, ignoring");
         }
         free(data);
         return;
@@ -1296,11 +1069,11 @@ void vdagent_x11_clipboard_data(struct vdagent_x11 *x11, uint8_t selection,
     if (type_from_event != type ||
             selection != x11->selection_req->selection) {
         if (selection != x11->selection_req->selection) {
-            SELPRINTF("expecting data for selection %d got %d\n",
+            SELPRINTF("expecting data for selection %d got %d",
                       (int)x11->selection_req->selection, (int)selection);
         }
         if (type_from_event != type) {
-            SELPRINTF("expecting type %u clipboard data got %u\n",
+            SELPRINTF("expecting type %u clipboard data got %u",
                       type_from_event, type);
         }
         vdagent_x11_send_selection_notify(x11, None, NULL);
@@ -1317,7 +1090,7 @@ void vdagent_x11_clipboard_data(struct vdagent_x11 *x11, uint8_t selection,
 
     if (size > x11->max_prop_size) {
         unsigned long len = size;
-        VSELPRINTF("Starting incr send of clipboard data\n");
+        VSELPRINTF("Starting incr send of clipboard data");
         x11->selection_req_data = data;
         x11->selection_req_data_pos = 0;
         x11->selection_req_data_size = size;
@@ -1343,14 +1116,14 @@ void vdagent_x11_clipboard_data(struct vdagent_x11 *x11, uint8_t selection,
 void vdagent_x11_clipboard_release(struct vdagent_x11 *x11, uint8_t selection)
 {
     XEvent event;
-    Atom clip;
+    Atom clip = None;
 
     if (vdagent_x11_get_clipboard_atom(x11, selection, &clip)) {
         return;
     }
 
     if (x11->clipboard_owner[selection] != owner_client) {
-        SELPRINTF("received release while not owning client clipboard\n");
+        SELPRINTF("received release while not owning client clipboard");
         return;
     }
 
